import datetime
from apist.email_ import send_email
from apist.log import logger
from apist.excel_ import ApiExcel
from apist.report import generate
from apist.request_ import Request
from apist.setting import CUT_RANGE, FLAG_NO, FLAG_YES, DEPENDED_FLAG
from requests.exceptions import ConnectTimeout

TIME1 = datetime.datetime.now()
log = logger()


class Api:
    def __init__(self, excel_file: str, sheet=0, mark=1, is_email=False):
        """
        excel_file: the excel file path
        mark: 0-运行所有，其它为运行所在优先级
        """
        log.info('start...')
        self.excel_file = excel_file
        self.req = Request()
        self.excel = ApiExcel(excel_file)
        if not isinstance(mark, int):
            mark = 1
        if not isinstance(sheet, int or str):
            sheet = 0
        self.rows = self.excel.get_rows(sheet=sheet)
        self.mark = mark
        self.is_email = is_email
        self.run()

    def run(self):
        """
        the thinking:
        1. do request from excel
        2. record message from response
        3. generate report by testing data
        4. email for information
        """
        all_cases, pass_cases, fail_cases, fake_fail_cases = self.do_req()
        self.do_res(all_cases)
        time2 = datetime.datetime.now()
        delta = time2 - TIME1
        generate(all_cases, pass_cases, fail_cases, fake_fail_cases, waste_time=delta)
        self.do_email(self.is_email)

    def do_req(self):
        """
        run excel case
        """
        # Get initial information
        e = self.excel
        req = self.req
        status_code_li = []
        all_cases = []
        pass_cases = []
        fail_cases = []
        fake_fail_cases = []
        headers = e.get_header()

        # Traverse all rows
        for i in range(1, self.rows):
            if e.get_run(i) == self.mark:
                method = e.get_method(i).lower()
                url = e.get_url(i)
                data = e.get_req_data(i)
                data = data if data else None

                # send request
                try:
                    if method == "get":
                        status_code, res_text = req.get(url, params=data, headers=headers)
                    elif method == "post":
                        status_code, res_text = req.post(url, data=data, headers=headers)
                    elif method == "put":
                        status_code, res_text = req.put(url, params=data, headers=headers)
                    elif method == 'delete':
                        status_code, res_text = req.delete(url, data=data, headers=headers)
                    else:
                        raise ValueError("must be get/post/put/delete")
                except ConnectTimeout:
                    status_code, res_text = 500, "ConnectTimeout"

                # save all if it has depended
                case_depend = self.excel.get_case_depend(i)
                if case_depend not in DEPENDED_FLAG:
                    res_text = res_text[:CUT_RANGE]

                pre_fail = e.get_pre_fail(i)
                expect = e.get_assert(i)
                if not expect:
                    if status_code == 200:
                        result = 'PASS'
                        flag = 1
                    elif pre_fail:
                        result = 'PASS'
                        flag = 2
                    else:
                        result = 'FAIL'
                        flag = 0
                else:
                    if expect in res_text:
                        result = 'PASS'
                        flag = 1
                    elif pre_fail:
                        result = 'PASS'
                        flag = 2
                    else:
                        result = 'FAIL'
                        flag = 0

                status_code_li.append(status_code)
                explain = e.get_explain(i)

                req_data = data if data else ''
                detail_request_data = '请求参数：' + req_data + '\n'
                detail_response = '接口响应：' + res_text + '\n'

                # Avoid interpreting labels
                detail_response = detail_response.replace('<', '&lt;')
                detail_response = detail_response.replace('>', '&gt;')
                detail = detail_request_data + detail_response

                case_all = [str(i), explain, url, status_code, expect, result, detail]
                all_cases.append(case_all)
                if flag == 0:
                    case_fail = [str(i), explain, url, status_code, expect, result, detail]
                    fail_cases.append(case_fail)
                elif flag == 2:
                    case_pre = [str(i), explain, url, status_code, expect, result, detail]
                    fake_fail_cases.append(case_pre)
                else:
                    case_pre = [str(i), explain, url, status_code, expect, result, detail]
                    pass_cases.append(case_pre)

        # for report
        for i in (all_cases, pass_cases, fail_cases, fake_fail_cases):
            for j in i:
                j.insert(-1, "详细")

        return all_cases, pass_cases, fail_cases, fake_fail_cases

    def do_res(self, cases: list):
        """
        do something from response
        """
        success_num = fail_num = 0
        fail_urls = []
        fail_status = []
        for _ in cases:
            id_, explain, url, s_code, expect, result, no_use, detail = _

            case_depend = self.excel.get_case_depend(id_)
            star = detail.index('接口响应：') + 5
            res = detail[star:]
            if case_depend != '!' or case_depend != '！':
                res = detail[star:CUT_RANGE]

            self.excel.write_status_code(id_, s_code)
            self.excel.write_response(id_, res)
            if result == 'PASS':
                log.info('%d \t%s' % (s_code, url))
                success_num += 1
                color_index = 3
            else:
                log.error('%d \t%s' % (s_code, url))
                color_index = 4
                fail_num += 1

            self.excel.write_result(id_, result, color_index)

        all_num = success_num + fail_num
        all_num = all_num if all_num else 1

        # success rate and failure rate
        success_rate, failure_rate = round(
            success_num / all_num, 4) * 100, round(fail_num / all_num, 4) * 100

        log.info("执行完毕...\n\n本次共执行 %d条用例，通过%d条，失败%d条；成功率为%d%%, 失败率为%d%%" % (
            all_num, success_num, fail_num, success_rate, failure_rate))
        return success_num, fail_num, fail_urls, fail_status

    def do_email(self, is_email):
        """
        check field and call send
        """
        if not is_email:
            return

        email_dict = self.excel.email_sheet()

        # weather run
        is_email2 = email_dict.get('run')
        if not is_email2 or is_email2 in FLAG_NO:
            log.info("the email run is an empty string, do not send email")
            return
        elif is_email2 not in FLAG_YES:
            log.error("ValueError: the email run must in %s" % (FLAG_YES + FLAG_NO))
            return

        # msg for email
        name = email_dict.get('name')
        pd = email_dict.get('password')
        to = email_dict.get('to')
        tittle = email_dict.get('tittle')
        content = email_dict.get('content')
        attachment = email_dict.get('attachment')

        if not (name and pd and to):
            log.error("email name or password missing")
            return False

        if attachment and '.' not in attachment:
            log.error("wrong file type %s" % attachment)
            return

        return send_email(name, pd, to, tittle, content, attachment)
