import xlwt
import xlrd
from xlutils.copy import copy
from apist.setting import RELY_EXPRESS, RUN, EXPLAIN, URL, REQ_DATA, STATUS_CODE, \
    RESPONSE, CASE_DEPEND, ASSERT, RESULT, METHOD, PRE_FAIL, DEPENDED_FLAG
from xlwt.Style import default_style


class Excel:
    def __init__(self, excel_file, sheet_name=''):
        self.excel_file = excel_file
        self.book = xlrd.open_workbook(excel_file, formatting_info=True)

        # open the table with the specified name
        try:
            if sheet_name:
                self.sheet_ = self.book.sheet_by_name(sheet_name=sheet_name)
                self.sheet_name_index = self.book.sheet_names().index(sheet_name)
            else:
                self.sheet_ = self.book.sheet_by_index(0)
                self.sheet_name_index = 0
        except xlrd.biffh.XLRDError:
            print("no such table %s' in excel" % sheet_name)
        self.book_copy = copy(self.book)
        self.sheet = self.book_copy.get_sheet(self.sheet_name_index)

    def cell_value(self, row: int, col):
        value = self.sheet_.cell_value(int(row), col)
        return value

    def get_run(self, row: int):
        run = self.cell_value(row, RUN)
        try:
            run = int(run)
        except ValueError:
            print("RUN must be a number")
        return run

    def get_explain(self, row: int):
        return self.cell_value(row, EXPLAIN)

    def get_method(self, row: int):
        method: str = self.cell_value(row, METHOD)
        if method.lower() not in ["get", "post", "put", "delete"]:
            raise ValueError("method invalid")
        return method

    def get_url(self, row: int):
        url: str = self.cell_value(row, URL)
        if not url.startswith("http"):
            raise ValueError("url invalid")
        return url

    def get_req_data(self, row: int):
        return self.cell_value(row, REQ_DATA)

    def get_status_code(self, row: int):
        return self.cell_value(row, STATUS_CODE)

    def get_response(self, row):
        return self.cell_value(row, RESPONSE)

    def get_case_depend(self, row: int):
        row = int(row)
        case_depend = self.cell_value(row, CASE_DEPEND)
        if not case_depend:
            return False

        if case_depend in DEPENDED_FLAG:
            return case_depend  # it might be '!' or '', that means be relied
        try:
            case_depend = int(float(case_depend))
        except ValueError:
            raise ValueError(("case depend must be int or !, the wrong row %s " % row))

        if case_depend > row:
            print("warn: depended row %s must at the top of row %s" % (row, case_depend))
            return False
        return case_depend

    def get_rely_express(self, row):
        if not self.get_case_depend(row):
            raise ValueError("case depend invalid")
        return self.cell_value(row, RELY_EXPRESS)

    def get_assert(self, row):
        return self.cell_value(row, ASSERT)

    def get_pre_fail(self, row):
        pre_fail = self.cell_value(row, PRE_FAIL)
        if pre_fail:
            if pre_fail not in ['1', 'yes', 'YES', 'Yes']:
                raise ValueError("pre fail invalid, it must in ['1', 'yes', 'YES', 'Yes']")
        return pre_fail

    def get_result(self, row):
        return self.cell_value(row, RESULT)

    def get_columns(self, sheet='', row=0):
        return self.all_columns(sheet=sheet, row=row)

    def get_rows(self, sheet=0):
        return self.all_rows(sheet=sheet)

    def get_email(self):
        return self.email_sheet()

    def get_header(self):
        return self.header_sheet()


class ApiExcel(Excel):
    def email_sheet(self) -> dict:
        if "email" not in self.book.sheet_names():
            print("no such sheet 'email'")
            return {}
        es = self.book["email"]
        cols = self.all_columns(es)
        values = self.all_columns(es, row=1)
        email_dict = {}
        for i, j in zip(cols, values):
            email_dict[i] = j
        return email_dict

    def header_sheet(self):
        if "header" not in self.book.sheet_names():
            print("no such sheet 'header'")
            return None
        hs = self.book["header"]
        cols = self.get_columns(hs)
        values = self.get_columns(hs, row=1)
        headers = {}
        for i, j in zip(cols, values):
            headers[i] = j
        return headers

    def all_columns(self, sheet='', row=0):
        columns = []
        sheet = sheet if sheet else self.sheet_
        try:
            for i in range(20):
                col = sheet.cell_value(row, i)
                if col:
                    columns.append(col)
        except IndexError:
            pass
        return columns

    def all_rows(self, sheet=0):
        sheet = sheet if sheet else self.sheet_
        return sheet.nrows

    @staticmethod
    def gen_style(cell_style=1):
        """
        Beautification of Excel form style
        """
        style, al, font = xlwt.XFStyle(), xlwt.Alignment(), xlwt.Font()
        if cell_style == 1:
            style.alignment.wrap = 1  # wrap automatically
        elif cell_style == 2:
            style.alignment.wrap = 1  # wrap automatically
            font.colour_index = 4
        elif cell_style == 3:
            font.bold = True
            font.colour_index = 3  # set green color
        elif cell_style == 4:
            font.bold = True
            font.colour_index = 2  # set red color
        else:
            style.alignment.wrap = 1
        style.alignment = al
        style.font = font
        style.alignment.vert = 0x01  # set vertical center
        style.alignment.horz = 0x02  # set horizontal center
        # to see the details by "print(style.alignment.vert, style.font.bold)"
        return style

    def write_cell(self, row, col, value: str, style=default_style):
        self.sheet.write(int(row), col, str(value), style)
        self.book_copy.save(self.excel_file)

    def write_response(self, row, value):
        """
        write response to excel
        """
        return self.write_cell(row, RESPONSE, value)

    def write_status_code(self, row, status_code):
        """
        write status code with style to excel
        """
        row = int(row)
        if status_code == 200:
            color = 3
        else:
            color = 4
        return self.write_cell(row, STATUS_CODE, status_code, self.gen_style(color))

    def write_result(self, row, result, color_index: int):
        """
        write result to excel
        """
        return self.write_cell(row, RESULT, result, self.gen_style(color_index))


if __name__ == '__main__':
    ae = ApiExcel("../api.xls")
    print(ae.get_header())
